"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reflect_1 = require("@plumjs/reflect");
const fs_1 = require("fs");
const path_1 = require("path");
String.prototype.format = function (...args) {
    return this.replace(/{(\d+)}/g, (m, i) => typeof args[i] != 'undefined' ? args[i] : m);
};
Array.prototype.flatten = function () {
    return this.reduce((a, b) => a.concat(b), []);
};
function hasKeyOf(opt, key) {
    return key in opt;
}
exports.hasKeyOf = hasKeyOf;
function isCustomClass(type) {
    switch (Array.isArray(type) ? type[0] : type) {
        case Boolean:
        case String:
        case Array:
        case Number:
        case Object:
        case Date:
            return false;
        default:
            return true;
    }
}
exports.isCustomClass = isCustomClass;
function resolvePath(path) {
    const removeExtension = (x) => x.replace(/\.[^/.]+$/, "");
    if (fs_1.existsSync(`${path}.js`))
        return [removeExtension(path)];
    else if (fs_1.existsSync(`${path}.ts`))
        return [removeExtension(path)];
    //resolve provided path directory or file
    else if (fs_1.lstatSync(path).isDirectory()) {
        const files = fs_1.readdirSync(path)
            //take only file in extension list
            .filter(x => [".js", ".ts"].some(ext => path_1.extname(x) == ext))
            //add root path + file name
            .map(x => removeExtension(x))
            .map(x => path_1.join(path, x));
        return Array.from(new Set(files));
    }
    else
        return [path];
}
exports.resolvePath = resolvePath;
function reflectPath(path) {
    if (Array.isArray(path))
        return path.map(x => reflect_1.reflect(x));
    else if (typeof path === "string")
        return resolvePath(path)
            .map(x => reflect_1.reflect(x))
            .map(x => x.members)
            .flatten();
    else
        return [reflect_1.reflect(path)];
}
exports.reflectPath = reflectPath;
const log = console.log;
var consoleLog;
(function (consoleLog) {
    function startMock() {
        console.log = jest.fn(message => { });
    }
    consoleLog.startMock = startMock;
    function clearMock() {
        console.log = log;
    }
    consoleLog.clearMock = clearMock;
})(consoleLog = exports.consoleLog || (exports.consoleLog = {}));
function getChildValue(object, path, defaultValue) {
    return path
        .split(/[\.\[\]\'\"]/)
        .filter(p => p)
        .reduce((o, p) => o ? o[p] : defaultValue, object);
}
exports.getChildValue = getChildValue;
